#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
A simple script for simplifying RPython JIT traces.

Example usage: python pypytrace.py example.pypylog
"""
from __future__ import print_function
import sys


def simplify(lines):
    remove = {'pyhaskell.interpreter.haskell.',
              'pyhaskell.interpreter.primtype.',
              'pyhaskell.interpreter.module.'}

    output = []
    for line in lines:
        # Remove '+33: ' etc
        if line.startswith('+'):
            line = line.split(':', 1)[1]

        # Remove some basic patterns
        for match in remove:
            if match in line:
                line = line.replace(match, '')

        # Remove some object references
        if ' object at 0x' in line:
            words = line.split(' object at 0x')
            for i in range(1, len(words)):
                words[i] = '>' + words[i].split('>', 1)[1]
            line = ''.join(words)

        output.append(line.strip())
    return '\n'.join(output)


def count(text):
    """Count operations and guards."""
    ops = guards = 0
    for line in text.split('\n'):
        if line.startswith('# Loop') or line.startswith('# bridge'):
            ops += int(line.rsplit('with', 1)[1].split(' ')[1])
        elif line.startswith('+'):
            line = line.split(':', 1)[1].strip()  # Remove '+33: ' etc
        if line.startswith('guard'):
            guards += 1
    return ops, guards


if __name__ == '__main__':
    if len(sys.argv) < 2:
        print("Error, missing path of input file.")
        print(__doc__)
        sys.exit(2)

    if len(sys.argv) == 3 and sys.argv[2] == 'count':
        with open(sys.argv[1], 'r') as f:
            ops, guards = count(f.read())
        print("Operations count: %i" % ops)
        print("Guards count: %i" % guards)

    with open(sys.argv[1], 'r') as f:
        print(simplify(f.readlines()))
