"""
Some helper-functions for builtin libraries and their tests.
"""
from pyhaskell.interpreter.haskell import Symbol, make_constructor
from pyhaskell.interpreter.primtype import Int, Char


def create_list(lst):
    """Returns a Haskell head-tails list from a Python list."""
    cons = Symbol.get_symbol(':')
    current = make_constructor(Symbol.get_symbol("[]"), [])
    for element in reversed(lst):
        if isinstance(element, str):
            assert len(element) == 1
            element = Char(element)
        elif isinstance(element, int):
            element = Int(element)
        current = make_constructor(cons, [element, current])
    return current


def check_same_list(one, two):
    """Check that two head-tail lists are equal."""
    while True:
        if one.numargs() != two.numargs():
            return False

        if one.numargs() == 0 or two.numargs() == 0:
            break

        if one.getarg(0) != two.getarg(0):
            return False

        one = one.getarg(1)
        two = two.getarg(1)

    return True
