# -*- coding: utf-8 -*-
from pyhaskell.interpreter.module import CoreMod, expose_var
from pyhaskell.interpreter.haskell import Value, Symbol, make_constructor
from pyhaskell.interpreter.primtype import Int


mod = CoreMod("base:GHC.List")


@expose_var(mod, "take", [Int, Value])
def take(num, lst):
    """Take first 'num' elements from 'lst'."""
    stack = []
    current = lst
    for i in range(num.value):
        if current.numargs() == 2:
            stack.append(current.getarg(0))
            current = current.getarg(1)
        else:
            break

    cons = Symbol.get_symbol(":")
    current = make_constructor(Symbol.get_symbol("[]"), [])
    for i in range(len(stack) - 1, -1, -1):
        current = make_constructor(cons, [stack[i], current])
    return current


@expose_var(mod, "length", [Value])
def length(lst):
    """Returns the number of elements in 'lst'."""
    current = lst
    i = 0
    while current.numargs() > 0:
        i += 1
        current = current.getarg(1)
    return Int(i)


@expose_var(mod, "replicate", [Int, Value])
def replicate(num, a):
    """Create a list of 'num' elements of repeated 'a'."""
    cons = Symbol.get_symbol(':')
    current = make_constructor(Symbol.get_symbol("[]"), [])
    for i in range(num.value):
        current = make_constructor(cons, [a, current])
    return current


@expose_var(mod, "!!", [Value, Int])
def index(lst, a):
    """Returns the element in 'a' position of 'lst'."""
    current = lst
    i = a.value
    while current.numargs() > 0:
        if i <= 0:
            return current.getarg(0)
        i -= 1
        current = current.getarg(1)
    return make_constructor(Symbol.get_symbol("()"), [])
