# -*- coding: utf-8 -*-
"""
Module for primitive Haskell types.
"""
from pyhaskell.interpreter.haskell import (
    Value, Symbol, make_constructor, DEFINITE_MATCH, NEEDS_HNF, NO_MATCH)


# Booleans
true = make_constructor(Symbol.get_symbol("True"), [])
false = make_constructor(Symbol.get_symbol("False"), [])


class Char(Value):
    _immutable_fields_ = ["value"]

    def __init__(self, char):
        assert isinstance(char, str)
        assert len(char) == 1
        self.value = char

    def tostr(self):
        return self.value


class Int(Value):
    _immutable_fields_ = ["value"]

    def __init__(self, integer):
        assert isinstance(integer, int)
        self.value = integer

    def match(self, other, subst):
        value = other.getvalue()
        if value:
            assert isinstance(value, Int)
            if self.value == value.value:
                return DEFINITE_MATCH
            return NO_MATCH
        return NEEDS_HNF

    def __eq__(self, other):
        return (isinstance(other, Int) and self.value == other.value)

    def __ne__(self, other):
        return not (self == other)

    def tostr(self):
        return str(self.value)  # TODO??


class Addr(Value):
    _immutable_fields_ = ["value"]

    def __init__(self, str_):
        assert isinstance(str_, str)
        self.value = str_

    def tostr(self):
        return self.value


class Double(Value):
    _immutable_fields_ = ["value"]

    def __init__(self, double):
        assert isinstance(double, float)
        self.value = double

    def tostr(self):
        return str(self.value)  # TODO??


class Float(Value):
    _immutable_fields_ = ["value"]

    def __init__(self, float_):
        assert isinstance(float_, float)
        self.value = float_

    def tostr(self):
        return str(self.value)  # TODO??
