#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import sys

from pyhaskell.interpreter import jscparser, haskell, module, primtype, prim


# Import Haskell modules implemented in Python
from pyhaskell.builtin import (unit, types, show, cstring,
                               base, num, tophandler, io, hslist)


def _need_compilation(path1, path2, path3=None):
    """Checks if path1 exists and is up to date or need to be compiled."""
    try:
        f1_mtime = os.stat(path1).st_mtime
        if f1_mtime < os.stat(path2).st_mtime:
            return True
        if path3 is not None:
            if f1_mtime < os.stat(path3).st_mtime:
                return True
    except OSError:
        return True
    return False


def interp(path):
    main_module = module.CoreMod("main:Main")
    module.CoreMod.all_modules["main::Main"] = main_module

    js = jscparser.parse_js(path)
    jscparser.AST().get_ast(js)
    haskell.evaluate_hnf(main_module.qvars["main"])


def main(args):
    if len(args) < 2:
        print "Missing file path"
        return 1

    # Compile core2js if needed
    core2js = "pyhaskell/core/core2js"
    if _need_compilation(core2js, '%s.hs' % core2js):
        res = os.system('ghc %s' % core2js)
        if res:
            return 1
        os.unlink(core2js + '.o')
        os.unlink(core2js + '.hi')

    # Process and interpret files as needed
    for path in args[1:]:
        if '.' not in path:
            print "Unsupported path %s" % path
            return 1

        file_, ext = path.rsplit('.', 1)

        if ext not in ('hs', 'lhs', 'hcr', 'hcj'):
            print "Unsupported file type: %s" % path
            return 1

        # Create hcr file
        if ext in ('hs', 'lhs') and _need_compilation(file_ + '.hcr', path):
            res = os.system('ghc -fext-core -fforce-recomp %s.hs' % file_)
            if res:
                return 1
            os.unlink(file_)
            os.unlink(file_ + '.o')
            os.unlink(file_ + '.hi')

        # Create hcj file
        if ext != 'hcj' and _need_compilation(file_ + '.hcj', path, core2js):
            res = os.system('%s %s.hcr %s.hcj' % (core2js, file_, file_))
            if res:
                return 1

        interp(file_ + '.hcj')

    return 0


if __name__ == '__main__':
    main(sys.argv)
