#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Small utility for generating JSON-parsed trees of JSCore files (.hcj).
"""
import os
import sys

from pypy.rlib.parsing.ebnfparse import parse_ebnf, make_parse_function


EBNF = """
    STRING: "\\"[^\\\\"]*\\"";
    NUMBER: "\-?(0|[1-9][0-9]*)(\.[0-9]+)?([eE][\+\-]?[0-9]+)?";
    IGNORE: "[ \t\n]";
    value: <STRING> | <NUMBER> | <object> | <array> | <"null"> | <"true"> | <"false">;
    object: ["{"] (pair [","])* pair ["}"] | ["{}"];
    array: ["["] (value [","])* value ["]"] | ["[]"];
    pair: STRING [":"] value;
    """


def create_dot(path):
    regexs, rules, ToAST = parse_ebnf(EBNF)
    parse = make_parse_function(regexs, rules, eof=True)

    with open(path, 'r') as f:
        doc = f.read()
    t = parse(doc)
    t = ToAST().transform(t)

    return 'digraph parsed {\n%s\n}' % '\n'.join(list(t.dot()))


def main(script, path=''):
    if not path:
        print "missing file path to parse"
        sys.exit(2)

    # Create dot file
    name = os.path.splitext(path)[0]
    dotfile = '%s.tmp.dot' % name
    outfile = '%s.png' % name
    with open(dotfile, 'w') as f:
        f.write(create_dot(path))

    # Create png file
    os.system('dot -Tpng %s > %s' % (dotfile, outfile))
    os.unlink(dotfile)

    print "Created graph of '%s' in '%s'" % (path, outfile)


# If run directly, generate png graph
if __name__ == "__main__":
    main(*sys.argv)
